//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : debug module
// created (by) : Christian Behrenberg, 2007-10-10
//--------------------------------------------------------------------

	#include "deb.h" // Load header

//--------------------------------------------------------------------
// INITIALIZATION
//--------------------------------------------------------------------
void deb_init ()
{
	#ifndef COMPILE

		add_folder("debug");							// Add folder to engine filesystem
  		deb_fnt = font_create("deb_fnt.tga");	// Create font(s)

	  	// Create echolist
			debtextList = txt_create(30, 999999);

		// String initialization
			int i;
			for (i = 0; i < debtextList.strings; i++) {
				(debtextList->pstring)[i] = str_create("#256");
			}

		// Init text
			debtextList->font = deb_fnt;
			debtextList->pos_x = 40;
			debtextList->pos_y = 80;
			set(debtextList, VISIBLE);
	#endif
}

//--------------------------------------------------------------------
// ECHOLIST
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// CLEARS THE ECHOLIST WITH EMPTY STRINGS IN EACH ROW
	//-----------------------------------------------------------------
	void deb_clearList ()
	{
		#ifndef COMPILE

			int i;
			for (i = 0; i < debtextList->strings; i++) {
				str_cpy((debtextList->pstring)[i], "");
			}

		#endif
	}

	//-----------------------------------------------------------------
	// CONVOLUTES THE ECHOLIST
	//
	// params: text      - text object
	//         direction - direction of convolution. -1 = down, +1 = up
	//-----------------------------------------------------------------
	void deb_convoluteList (TEXT* text, int direction)
	{
		#ifndef COMPILE

			if (direction == 0) {return;}

			int i;

			// Push down
			if (direction > 0) {

				for (i = text.strings - 1; i > 0; i--) {
					str_cpy((text->pstring)[i], (text->pstring)[i-1]);
				}

			} else { // Pull up

				for (i = 0; i < text.strings - 1; i++) {
					str_cpy((text->pstring)[i], (text->pstring)[i+1]);
				}

			}
		#endif
	}

	//-----------------------------------------------------------------
	// PRINTS A CHAR MESSAGE INTO THE ECHOLIST
	//
	// params:
	//-----------------------------------------------------------------
	void deb_print(char* txt)
	{
		#ifndef COMPILE
			if (deb_fnt != 0) { //print it only, if font is created
	
				char buffer[128];
				
			 	deb_log(txt); //log debug str to acklog.txt
	 			sprintf(buffer, "%d: %s", total_frames, txt); //create entry
				deb_convoluteList(debtextList, 1); //push list down
				
				//copy new entry into the first row
				str_cpy((debtextList->pstring)[0], buffer);
			}
		#endif
	}

	//-----------------------------------------------------------------
	// CONVENIENCE FUNCTIONS
	// Uses overloading to print arbitrary data into the echolist
	//-----------------------------------------------------------------
	
		void deb_print (STRING* str) {
			#ifndef COMPILE
				deb_print(str->chars);
			#endif
		}
		
		void deb_print (int val) {
			#ifndef COMPILE
				char* buffer [64];
				sprintf(buffer, "%d", val);
				deb_print(buffer);
			#endif
		}

		void deb_print (var val) {
			#ifndef COMPILE
				str_for_num(deb_str, _in);
				deb_print(deb_str->chars);
			#endif
		}

		void deb_print (float val) {
			#ifndef COMPILE
				str_for_num(deb_str, val);
				deb_print(deb_str->chars);
			#endif
		}

		void deb_print(double val) {
			#ifndef COMPILE
				str_for_num(deb_str, val);
				deb_print(deb_str->chars);
			#endif
		}

	//-----------------------------------------------------------------
	// LOGS A CHAR SEQUENCE INTO THE ACKLOG.TXT
	//-----------------------------------------------------------------
	void deb_log (char* txt)
	{
		#ifndef COMPILE
			char buffer[128];
			sprintf(buffer, "\nfrm(%d) = %s", (int)(total_frames), txt); // Create entry
		  	diag(buffer); // Print to acklog.txt
		#endif
	}