//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : effects
// created (by) : Christian Behrenberg, 2007-10-24
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// HORIZONTAL FOG
// Made of sprite planes
//--------------------------------------------------------------------
void eff_horFog ()
{
	if (detail == DETAIL_LOW) {
		ent_remove(my);
		return;
	}

	set(my, PASSABLE | INVISIBLE);
	reset(my, DYNAMIC);

	while (1) {
		you = ent_create(my.type, my.x, eff_horFog_cloud);
		vec_set(you.scale_x, my.scale_x);
		vec_set(you.pan, my.pan);
		wait(-(5 + random(8)));
	}
}

	//- sprite function -----------------------------------------------
	void eff_horFog_cloud ()
	{
		fx_alpha();

		set(my, TRANSLUCENT | PASSABLE | BRIGHT);
		my.alpha = 0;

		vec_set(vecTemp, vector(random(30), 0, random(40)));
		vec_rotate(vecTemp, vector(random(360), 0, 0));
		vec_add(my.x, vecTemp);

		my.pan = random(360);

		wait(1);

		vec_scale(my.scale_x, 1.1-random(0.2));

		while (my.alpha < 100)
		{
			my.alpha += 2 * time_step;
			my.z += 0.5 * time_step;
			my.pan += 0.25 * time_step;
			wait(1);
		}

		while (my.alpha > 0)
		{
			my.alpha -= 0.15 * time_step;
			my.z += 0.5 * time_step;
			my.pan -= 0.5 * time_step;
			wait(1);
		}

		ent_remove(my);
	}


//--------------------------------------------------------------------
// CREATES A SNOW-PARTICLE CLUSTER
// takes a weather box (size) and a number of particles (density) to
// create a good looking snow effect
//--------------------------------------------------------------------
void eff_snow_create (VECTOR* size, var density)
{
	vec_set(eff_weatherbox, size);
	vec_scale(eff_weatherbox, 0.5);

	//create the particles
	effect(eff_snow, density, nullvector, nullvector);
}

	//-----------------------------------------------------------------
	// SNOW PARTICLE
	// modulates in the weatherbox and flies with the wind
	//-----------------------------------------------------------------
	void eff_snow (PARTICLE *p)
	{
		//create a random position in the weatherbox
		vec_set(p.x, vector( camera.x+random(eff_weatherbox.x*2) - eff_weatherbox.x,
									camera.y+random(eff_weatherbox.y*2) - eff_weatherbox.y,
									camera.z+random(eff_weatherbox.z*2) - eff_weatherbox.z));

		p.bmap = bmapCheck(eff_snowFlakes_bmap, "effSnowFlakes.dds");
		p.size = 750 + random(800);
		set(p, MOVE);

		p.event = eff_snow_func;
	}

		//- particle function ---------------------------------------------
		void eff_snow_func (PARTICLE* p)
		{
			//keep the particle within the box
			vec_set(p->x, vector(cycle(p->x, camera.x - eff_weatherbox.x, camera.x + eff_weatherbox.x),
										cycle(p->y, camera.y - eff_weatherbox.y, camera.y + eff_weatherbox.y),
										cycle(p->z, camera.z - eff_weatherbox.z, camera.z + eff_weatherbox.z)));

			//update the movement speed and direction
			vec_set(p->vel_x, vector(eff_wind_dir.x * time_step, eff_wind_dir.y * time_step, eff_wind_dir.z * time_step));

			p.lifespan = 100; // live forever
		}

	//--------------------------------------------------------------------
	// SNOW TEMPLATES
	//--------------------------------------------------------------------

		void eff_snow_medium ()
		{
			vec_set(eff_wind_dir, vector(10,10,-25));
			eff_snow_create(vector(2500,2500,2250), 50);
		}

		void eff_snow_light ()
		{
			vec_set(eff_wind_dir, vector(10,10,-25));
			eff_snow_create(vector(2000,2000,2000), 25);
		}

		void eff_snow_verylight ()
		{
			vec_set(eff_wind_dir, vector(10,10,-25));
			eff_snow_create(vector(1800,1800,1800), 12);
		}

//--------------------------------------------------------------------
// PUFF EFFECT
//--------------------------------------------------------------------
void eff_puff (PARTICLE *p)
{
	p.bmap = bmapCheck(eff_puff_bmap, eff_puff_file);
	p.size = 32 + random(64);
	p.alpha = 25+random(25);
	p.lifespan = 100;
	set(p, TRANSLUCENT | MOVE);

	//displace start position

		vec_set(p.skill_x, vector(16 + random(48),0,0));
		vec_rotate(p.skill_x, vector(random(360), random(90), 0));
		vec_add(p.x, p.skill_x);

	//velocity

		vec_set(p.vel_x, vector(5 + random(8),0,0));
		vec_rotate(p.vel_x, vector(random(360), random(90), 0));

	p.event = eff_puff_func;
}

	//- particle function ---------------------------------------------
	void eff_puff_func (PARTICLE* p)
	{
		p.alpha -= 5 * time_step;				//fade out
		p.size += 2 * time_step;				//grow
		p.lifespan = (p.alpha > 0) * 100;	//survive if visible
	}

//--------------------------------------------------------------------
// MAGICAL SPARK EFFECT
//--------------------------------------------------------------------
void eff_sparkMagic(PARTICLE* p)
{
	p.bmap = bmapCheck(eff_sparkMagic_bmap, eff_sparkMagic_file);
	p.size = 16 + random(8);

	// Velocity
		vec_set(p.vel_x, vector(5 + random(8),0,0));
		vec_rotate(p.vel_x, vector(random(360), random(90), 0));

	my.alpha = 100;

	set(p, TRANSLUCENT | BRIGHT | MOVE);

	p.lifespan = 50;
	p.event = eff_sparkMagic_func;
}

	//- particle function ---------------------------------------------
	void eff_sparkMagic_func (PARTICLE* p)
	{
		p.alpha -= 5 * time_step;						//fade out
		p.size = maxv(8, p.size - 1 * time_step); //shrink, at least 1
		p.lifespan = (p.alpha > 0) * 100;			//survive if visible
	}
	
//--------------------------------------------------------------------
// SNOW SPLATTER
//--------------------------------------------------------------------
void eff_snowSplat (PARTICLE* p)
{
	p.bmap = bmapCheck(eff_snowSplat_bmap, eff_snowSplat_file);
	p.size = 24 + random(150);

	// Velocity
		vec_set(p.vel_x, vector(0, 0, 5 + random(8)));

	set(p, MOVE | TRANSLUCENT);

	p.lifespan = 100;
	p.alpha = 75;
	p.event = eff_snowSplat_func;
}

	//- particle function ---------------------------------------------
	void eff_snowSplat_func (PARTICLE* p)
	{
		p.vel_z -= 1.5 * time_step;
		p.alpha -= 4 * time_step;
		if (p.alpha <= 0) {p.lifespan = 0;}
	}

//--------------------------------------------------------------------
// BRIGHT SPARKS
//--------------------------------------------------------------------
void eff_sparkBright (PARTICLE* p)
{
	p.bmap = bmapCheck(eff_sparkBright_bmap, eff_sparkBright_file);
	p.size = 16 + random(8);

	//velocity

		vec_set(p.vel_x, vector(10 + random(10),0,0));
		vec_rotate(p.vel_x, vector(random(360), random(180), 0));

	my.alpha = 100;

	set(p, TRANSLUCENT | BRIGHT | MOVE);

	p.lifespan = 50;
	p.event = eff_sparkBright_func;
}

	//- particle function ---------------------------------------------
	void eff_sparkBright_func (PARTICLE* p)
	{
		p.alpha -= 10 * time_step;						//fade out
		p.size += 2 * time_step; //grow
		p.lifespan = (p.alpha > 0) * 100;			//survive if visible
	}

//--------------------------------------------------------------------
// CREATES PARTICLES AT VERTICES OF AN ENTITY
//
// params: ent - entity ("hull")
//         nr  - # of particles
//         fnc - particle function
//--------------------------------------------------------------------
void eff_spawnEnt_verts (ENTITY* ent, int nr, void* fnc)
{
	int i;
	for (i = 0; i < nr; i++) {
		vec_for_vertex(vecTemp.x, ent, random(ent_vertices(ent)));
		effect(fnc, 1, vecTemp, nullvector);
	}
}

//--------------------------------------------------------------------
// CREATES A GINGERBREAD HEART AT A POSITION
//
// params: pos - position vector
//--------------------------------------------------------------------
void eff_sledgeHeart3D (VECTOR* pos)
{
	ent_create("heartB.mdl", pos, eff_sledgeHeart3D_func);
}

	//- entity function -----------------------------------------------
	void eff_sledgeHeart3D_func ()
	{
		set(my, PASSABLE);

		//effect: spark
			eff_spawnEnt_verts(my, 12, eff_sparkMagic);

		my.skill1 = 1.75 * 16;			// Time
		my.skill2 = 15 + random(15);	// Pan speed
		my.skill3 = 16;					// Z-raise speed
		my.skill4 = random(360);		// Seed (scaling)

		if (my.pan == 0) {
			my.pan = random(360);
		}

		while (my.skill1 > 0) {

			my.pan += my.skill2 * time_step;

			my.z += my.skill3 * time_step;
			my.skill3 *= 0.8 * time_step;

			my.skill1 -= time_step;

			my.scale_y = my.scale_z = my.scale_x = 1 + sin((total_ticks + my.skill4) * 8) * 0.25;

			wait(1);
		}

		//effect: spark
			eff_spawnEnt_verts(my, 12, eff_sparkMagic);

		ent_remove(my);
	}

//--------------------------------------------------------------------
// LIGHT FLARE SPRITE
//--------------------------------------------------------------------
void eff_flare ()
{
	set(my, PASSABLE | TRANSLUCENT | BRIGHT);
	vec_set(my.pan, nullvector);
	my.alpha = 36;
	reset(my, DYNAMIC);
}

//--------------------------------------------------------------------
// STATIC SHADOW SPRITE
//--------------------------------------------------------------------
void eff_shadStatic ()
{
	set(my, PASSABLE);
	reset(my, DYNAMIC);
}

//--------------------------------------------------------------------
// GLOWING SPRITE FOR RUDIS NOSE
//--------------------------------------------------------------------
void eff_rudiNoseGlow ()
{
	ent_create("effRudiNoseGlow.dds", vector(0,0,-1000), eff_rudiNoseGlow_func);
}

	//- sprite function -----------------------------------------------
	void eff_rudiNoseGlow_func ()
	{
		set(my, PASSABLE | TRANSLUCENT | BRIGHT);

		while (1) {

			if (Rudi) {
				reset(my, INVISIBLE);
				vec_for_vertex(my.x, Rudi, 223);

				if (Rudi->playerDeath != 0) {
					break;
				}

			} else {
				set(my, INVISIBLE);
			}

			my.scale_x = my.scale_y = 1 + sin(total_ticks) * 0.5;
			my.alpha = 40 + sin(total_ticks * 4) * 5;

			wait(1);
		}

		ent_remove(my);
	}

//--------------------------------------------------------------------
// BIG FIRE EFFECT
//--------------------------------------------------------------------

	//--------------------------------------------------------------------
	// FIRE EFFECT
	//--------------------------------------------------------------------
	void eff_bigFire_fire (PARTICLE *p)
	{
		vec_set(p.vel_x, vector(random(2) - 1, random(2) - 1, random(2) + 1));
		p.alpha = 40 + random(20);
		p.bmap = bmapCheck(eff_bigFire_fireBmap, "eff_light_fire.dds");
		p.size = 60 + random(50);
		set(p, MOVE | BRIGHT);
		p.lifespan = 100;
		p.event = eff_bigFire_fire_func;
	}

		//- particle function ---------------------------------------------
		void eff_bigFire_fire_func (PARTICLE *p)
		{
			p.size -= 0.5 * time_step * (p.size > 1);
			p.alpha -= 1 * time_step;
			p.lifespan = (p.alpha > 0) * 10;
		}

	//--------------------------------------------------------------------
	// SMOKE EFFECT
	//--------------------------------------------------------------------
	void eff_bigFire_smoke (PARTICLE *p)
	{
		p.z += 50 + random(50);
		vec_set(p.vel_x, vector(random(1) - 0.5, random(1) - 0.5, random(4) + 2));
		p.alpha = 50 + random(10);
		p.bmap = bmapCheck(eff_bigFire_smokeBmap, "eff_light_smoke.dds");
		p.size = random(80)+20;
		set(p, MOVE);
		p.lifespan = 100;
		p.event = eff_bigFire_smoke_func;
	}

		//- particle function ---------------------------------------------
		void eff_bigFire_smoke_func (PARTICLE *p)
		{
			p.size		-= time_step * (p.size > 1);
			p.alpha 		-= 1 * time_step;
			p.lifespan  = (p.alpha > 0) * 10;

			p.red		= p.alpha * 2.5;
			p.green	= p.red;
			p.blue	= p.red;
		}


//--------------------------------------------------------------------
// FAKED SOFTSHADOW
//
// creates a blob sprite on the bbox ground (no trace!) on the basis
// of a vertex as postion. If you attach several blob-sprites with
// this method, it looks like a (faked) softshadow.
//
// params: ent    = shadowentity
//         vertex = vertex nr. to be attached
//         scale  = scale of the sprite
//--------------------------------------------------------------------
void effShadSoft(ENTITY* ent, int vertex, var scale)
{
	// Create spot
		ENTITY* spot = ent_create("effShadowspot.dds", vector(0,0,-1000), effShadSoft_ent);

	// Scale it
		spot->scale_x = scale;
		spot->scale_y = scale;

	// Pass parameters
		spot->skill1 = ent;		// Attached entity
		spot->skill2 = vertex;	// Attached vertex
}

	//-----------------------------------------------------------------
	// SHADOW SPRITE ENTITY
	//-----------------------------------------------------------------
	void effShadSoft_ent ()
	{
		wait(3); // Wait for valid parameter values

		// Initialization
			set(my, PASSABLE | TRANSLUCENT); // Passable and not solid
			my.tilt = 90;							// Flat on the floor

		while (1) {

			you = (ENTITY*)(my.skill1);	// Get entity
			if (you) {                    // If it exists...

				// If the engine explicitely wants the shadow to
				// be destroyed, do it

					if (you.deleteShadow) {
						break;
					}

				// Get vertex and pose me there
					vec_for_vertex(my.x, you, my.skill2);

				// Move to the feet
					my.z = you.z + you.min_z + 10;
					//We add some quants, because otherwise the sprite would flicker

			} else { // Entity doesnt exists
				break;
			}

			wait(1);
		}

		// Remove shadow
			ent_remove(my);
	}

//--------------------------------------------------------------------
// BLOB SHADOW WITH C-TRACE
//--------------------------------------------------------------------
void effShadBlobTrace (ENTITY* ent, var scale, char* file)
{
	ENTITY* spot = ent_create(file, vector(0,0,-1000), effShadBlobTrace_ent);
	spot->scale_x = scale;
	spot->scale_y = scale;
	spot->skill1 = ent;
}

	//-----------------------------------------------------------------
	// SPRITE ENTITY
	//-----------------------------------------------------------------
	void effShadBlobTrace_ent ()
	{
		wait(3);

		set(my, PASSABLE | TRANSLUCENT);
		my.tilt = 90;

		while (1) {

			you = (ENTITY*)(my.skill1);
			if (you) {

				if (you.deleteShadow) {
					break;
				}

				vec_set(vecTemp.x, you.x);
				vecTemp.z -= 0.5 * (you.max_z - you.min_z);
				vec_set(my.x, vecTemp.x);
				vecTemp.z -= 10000;

				c_trace(my.x, vecTemp.x, IGNORE_ME | IGNORE_YOU | IGNORE_PASSABLE | IGNORE_SPRITES);

				vec_set(my.x, target.x);
				my.pan = you.pan;
				my.z += 5;

			} else {
				break;
			}

			wait(1);
		}

		ent_remove(my);
	}

//-----------------------------------------------------------------
// BLOB SHADOW AT THE BOTTOM AT THE BBOX
//-----------------------------------------------------------------
void effShadBlob (ENTITY* ent, var scale, char* file)
{
	ENTITY* spot = ent_create(file, vector(0,0,-1000), effShadBlob_ent);
	spot->scale_x = scale;
	spot->scale_y = scale;
	spot->skill1 = ent;
}

	//-----------------------------------------------------------------
	// SPRITE ENTITY
	//-----------------------------------------------------------------
	void effShadBlob_ent ()
	{
		wait(3);
		
		proc_mode = PROC_LATE;

		set(my, PASSABLE | TRANSLUCENT);
		my.tilt = 90;

		while (1) {

			you = (ENTITY*)(my.skill1);
			if (you) {

				if (you.deleteShadow) {
					break;
				}

				vec_set(my.x, you.x);
				my.z += you.min_z + 5;
				my.pan = you.pan;

			} else {
				break;
			}

			wait(1);
		}

		ent_remove(my);
	}