//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : environment
// created (by) : Christian Behrenberg, 2007-11-12
//--------------------------------------------------------------------

//SKY

	//--------------------------------------------------------------------
	// SKYCUBE STARTUP FUNCTION
	// Creates skycubes on startup of the engine
	//--------------------------------------------------------------------
	void env_skycubes_startup ()
	{
		sky_northpole = ent_createlayer(FX_CUBE_SKY_NORTHPOLE, SKY | CUBE, 10);
	}

	//--------------------------------------------------------------------
	// LOADS A SKYCUBE
	// 
	// params: sky - preexisting sky entity
	//--------------------------------------------------------------------
	void env_loadSky (ENTITY* sky)
	{
		sky_cube_level = sky;
	}

//LANDSCAPE

	//--------------------------------------------------------------------
	// GROUND
	//--------------------------------------------------------------------
	void env_ground ()
	{
		if (detail >= DETAIL_MEDIUM) {
			fx_detailShiny();
		} else {
			fx_detail();
		}

		set(my, POLYGON);
		reset(my, DYNAMIC);
	}
	
		void env_groundColl ()
		{
			set(my, POLYGON | INVISIBLE);
			reset(my, DYNAMIC);
		}

	//--------------------------------------------------------------------
	// ROCKS
	//--------------------------------------------------------------------
	void env_rock ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}

//ICE

	//--------------------------------------------------------------------
	// ICEBERG
	//--------------------------------------------------------------------
	void env_iceberg ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);

		if (detail == DETAIL_HIGH) {
			fx_shiny(fx_cube_shiny);
		}
	}

//WATER

	//--------------------------------------------------------------------
	// ICE-WATER
	//--------------------------------------------------------------------
	void env_iceWater ()
	{
		fx_ocean(fx_cube_sky_northpole);

		set(my, TRANSLUCENT | PASSABLE);
		reset(my, DYNAMIC);
		my.alpha = 25;
	}

	//--------------------------------------------------------------------
	// WATER DEPTH PLANES
	// simulates water depth by multiple transparent planes
	//--------------------------------------------------------------------
	void env_waterDepth ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}

	//WATER OBJECTS

		//--------------------------------------------------------------------
		// SHINY ICE FLOE
		//--------------------------------------------------------------------
		void env_iceFloe ()
		{
			if (detail >= DETAIL_MEDIUM) {
				fx_shiny(fx_cube_shiny);
			}

			set(my, PASSABLE);
			reset(my, DYNAMIC);
		}

//PLANTS

	//--------------------------------------------------------------------
	// WEED / PLANTS
	//--------------------------------------------------------------------
	void env_weed ()
	{
		my.push = -1;
		fx_wavingGrass();
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}

	//--------------------------------------------------------------------
	// GENERAL TREE _WITHOUT_ COLLISION
	//--------------------------------------------------------------------
	void env_tree ()
	{
		set(my, PASSABLE | OVERLAY);
		reset(my, DYNAMIC);
	}

		//-----------------------------------------------------------------
		// GENERAL TREE _WITH_ COLLISION
		//-----------------------------------------------------------------
		void env_treeColl ()
		{
			set(my, OVERLAY);
			c_setminmax(my);

			vec_scale(my.min_x, 0.2);
			vec_scale(my.max_x, 0.2);
			my.min_z *= 10;
			my.max_z *= 10;

			reset(my, DYNAMIC);
		}
		
	//--------------------------------------------------------------------
	// TREE STUMP _WITHOUT_ COLLISION
	//--------------------------------------------------------------------
	void env_stump ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}

		//-----------------------------------------------------------------
		// TREE STUMP _WITH_ COLLISION
		//-----------------------------------------------------------------
		void env_stumpColl ()
		{
			c_setminmax(my);

			vec_scale(my.min_x, 0.75);
			vec_scale(my.max_x, 0.75);

			reset(my, DYNAMIC);
		}

//TRIGGERS

	//--------------------------------------------------------------------
	// SNOW EFFECT
	// starts a permanent snow effect (MEDIUM)
	//--------------------------------------------------------------------
	void env_snowMedium ()
	{
		switch (detail) {
			case DETAIL_LOW:		eff_snow_verylight(); break;
			case DETAIL_MEDIUM:	eff_snow_light(); break;
			case DETAIL_HIGH:		eff_snow_medium(); break;
		}

		ent_remove(my);
	}

	//--------------------------------------------------------------------
	// SKYCUBE LOADER
	// loads a skycube; the file is given via entity string __2__
	//--------------------------------------------------------------------
	void env_sky_northpole ()
	{
		env_loadSky(sky_northpole);
		ent_remove(my);
	}

	//--------------------------------------------------------------------
	// WIND SOUND EMITTER
	//--------------------------------------------------------------------
	void env_sndWind ()
	{
		set(my, INVISIBLE | PASSABLE);
		reset(my, DYNAMIC);

		while (1) {
			snd_env_windPlay();
			wait(1);
		}
	}

	//--------------------------------------------------------------------
	// SEASHORE SOUND EMITTER
	//--------------------------------------------------------------------
	void env_sndWaterWind ()
	{
		set(my, INVISIBLE | PASSABLE);
		reset(my, DYNAMIC);

		snd_emitByEntDist(my, pl_listener, 2500, snd_env_waterWindPlay, snd_env_waterWindStop);
	}

	//--------------------------------------------------------------------
	// WATER SOUND EMITTER
	//--------------------------------------------------------------------
	void env_sndWater ()
	{
		set(my, INVISIBLE | PASSABLE);
		reset(my, DYNAMIC);

		snd_emitByEntDist(my, pl_listener, 2500, snd_env_waterPlay, snd_env_waterStop);
	}

	//--------------------------------------------------------------------
	// SEAGULL SOUND EMITTER
	//--------------------------------------------------------------------
	void env_sndGulls ()
	{
		set(my, INVISIBLE | PASSABLE);
		reset(my, DYNAMIC);

		snd_emitByEntDist(my, pl_listener, 2500, snd_env_gullsPlay, snd_env_gullsStop);
	}