//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : game
// created (by) : Christian Behrenberg, 2007-10-10
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// GAME MAIN FUNCTION
// Initializes and runs the game
//--------------------------------------------------------------------
void game ()
{
	// Initialization
		game_init();

	// Splashscreens
		splash_show();
		while (!splash_ready) {wait(1);}

	// Menu
		menu_main();
}

	//--------------------------------------------------------------------
	// INITIALIZES THE GAME
	//--------------------------------------------------------------------
	void game_init ()
	{
		// Add game content folders to engine file system
			add_folder("game\\levels");	//level data
			add_folder("game\\effects");	//effects data
			add_folder("game\\hud");		//effects data
			add_folder("game\\splash");	//splashscreens
			add_folder("game\\menu");		//menu data
			add_folder("game\\credits");	//credits data
			add_folder("game\\sounds");	//sound data
	
		// Start packet arrow function
			pk_arrow_main();
	
		// Initialize HUD
			hud_main();
	
		// General game variables
			game_init_variables();
	
	}
	
		//- initializes the most important game variables -----------------
		void game_init_variables ()
		{
			volMusic	= 80;
			volSfx	= 95;
			detail	= DETAIL_MEDIUM;
			fps_max	= 60;
		}

//--------------------------------------------------------------------
// STARTS THE MAIN GAME
// Starts the first level
//--------------------------------------------------------------------
void game_start ()
{
	// Close the menu
		menu_hide_all();
		menu_mode = MENU_MODE_NONE;
		
	// Engine settings
		max_particles		= 4000;
		max_entities		= 1500;
		preload_mode		= 2;
		tree_range			= 700;
		camera.clip_far	= 14500;

		vec_set(d3d_lodfactor,vector(99,99,99));

	// Primary game reset
		lvl_current	= 1;					// Start first level
		lifes			= lifes_resetNr;	// Reset lifes
		points		= 0;              // Reset points

	// Load level
		level_load("northpole.wmb");

	// Level reset
		lvl_reset();
		music_level_start();

	// Synchronize
		wait(1);

	// Preload of secondary data
		lvl_preload();
}

//--------------------------------------------------------------------
// WATCHES POINTS TO ADD AND ADDS IT OVER AN INTERVAL
//--------------------------------------------------------------------
void game_watchPoints ()
{
	while (1) {

	// Add points, if requested
		if (points_toAdd > 0) {

			points++;
			points_toAdd--;

			wait(-0.012);
		}

		wait(1);
	}
}

//--------------------------------------------------------------------
// WATCHES LIFES
//--------------------------------------------------------------------
void game_watchLifes ()
{
	int original_lifes = lifes; //save the current lifes to recognize changes

	while (1) {
		if (lifes != original_lifes) {		//Recognize difference
			if (lifes >= 0) {						//we are still alive..
// 				if (lifes < original_lifes) {	//lost one life
// 					deb_print("lifes -1");
// 				} else {
// 					deb_print("lifes +1");		//one new life
// 				}

				original_lifes = lifes;			//reset checking

			} else {									//Rudi is dead
				lifes = 0;
			}
		}

		wait(1);
	}
}