//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : npc's
// created (by) : Christian Behrenberg, 2007-11-17
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// SEA GULLS
//--------------------------------------------------------------------
void npc_seagull ()
{
	if (detail == DETAIL_LOW) {
		ent_remove(my);
		return;
	}

	effShadBlobTrace(my, 2, "effShadSeaGull.dds");

	// Initialization

		npc_seagull_init();
		act_path_assign(my, my.string2); // Assign path

		var seed = random(30000);
		var speedSeed = 13 + random(10);

	// Main loop
	while (1) {

		// Motion
			act_path_run(my, speedSeed, seed);					 // Run path

		// Animation
			ent_animate(my, "fly", 10 * (total_ticks + seed), ANM_CYCLE);

		wait(1);
	}
}

	void npc_seagull_init ()
	{
		set(my, PASSABLE);
	}


//--------------------------------------------------------------------
// FISHES
//--------------------------------------------------------------------

	//- general fish initialization -----------------------------------

		void npc_fish_init ()
		{
			set(my, PASSABLE);
		}

	//-----------------------------------------------------------------
	// Fish mother. Swims along a path.
	//-----------------------------------------------------------------
	void npc_fishMom ()
	{
		if (detail == DETAIL_LOW) {
			ent_remove(my);
			return;
		}

		// Initialization

			npc_fish_init();						// Make me a fish!
			my.entID = ID_NPC_FISH_MOM;		// I am a mother
			act_path_assign(my, my.string2); // Assign path

		   var seed = random(10000);
		   var speedSeed = random(5);
			var originalZ = my.z;

		// Swim along the path
		while (1) {

			varTemp[0] = 12 + speedSeed;								// Base speed
			varTemp[0] += sin(total_ticks + seed) * 3;			// Variation 1
			varTemp[0] += sin((total_ticks + seed) * 2) * 3;	// Variation 2

			// Motion
				act_path_run(my, varTemp[0], seed);					 // Run path
				my.z = originalZ + sin(total_ticks + seed) * 20; // Float up & down

			// Animation
				ent_animate(my, "swim", (var)(my.skill90 * 0.25) % 100, ANM_CYCLE);

			wait(1);
		}
	}
	
	//-----------------------------------------------------------------
	// Fish kid. Is connected with a fish mother and follows her.
	//-----------------------------------------------------------------
	void npc_fishKid ()
	{
		if (detail == DETAIL_LOW) {
			ent_remove(my);
			return;
		} else {
			if ((detail == DETAIL_MEDIUM) && (my.skill1 % 2 == 1)) {
				ent_remove(my);
				return;
			}
		}

		// Initialization

			npc_fish_init();							// Make me a fish!
			wait(3);										// Wait for mom-ents
			act_fish_searchMom(my, my.skill1);	// Search my mom
			vec_scale(my.scale_x, 0.5);			// Make me smaller

			var seed = random(10000);
			var speedSeed = random(5);
			var originalZ = my.z;

		// Follow my mom
		while (1) {

			// Get mom
			you = (ENTITY*)(my.skill2);
			if (you) {

				// Rotate target point around my mother
					vec_set(vecTemp.x, vector(150, 0, 0));
					vec_rotate(vecTemp.x, vector(total_ticks + seed, 0, 0));
					vec_add(vecTemp.x, you.x);
					vec_diff(vecTemp.x, vecTemp.x, my.x);

				// Calculate speed

					varTemp[0] = 7 + speedSeed;								// Base speed
					varTemp[0] += sin(total_ticks + seed) * 3;			// Variation 1
					varTemp[0] += sin((total_ticks + seed) * 2) * 3;	// Variation 2

					vec_normalize(vecTemp, varTemp[0]);
					vec_scale(vecTemp, time_step);
					my.skill90 += vec_length(vecTemp);

				// Move
					vec_add(my.x, vecTemp);

				// Rotation
					vec_to_angle(vecTemp, vecTemp);
					ang_lerp(my.pan, my.pan, vecTemp.x, 0.2 * time_step);

				// Float p & down
					my.z = originalZ + sin(total_ticks + seed) * 20;

				// Animation
					ent_animate(my, "swim", (var)(my.skill90 * 0.25) % 100, ANM_CYCLE);
			}
	
			wait(1);
		}
	}

		//- finds a mother for a fish kid -------------------------------

			void act_fish_searchMom (ENTITY* kid, var search)
			{
				// Goes through the entity list and searches for a fish
				// mother (indicated by the ID_ACT_FISH_MOM entID). SEARCH
				// determines the connected mother. The mother's entity
				// pointer is saved in skill2 of the fish kid.

				you = ent_next(NULL);
				while (you != NULL) {

					if ((you.entID == ID_NPC_FISH_MOM) && (you.skill1 == search)) {
						kid->skill2 = you;
						break;
					}
			
					you = ent_next(you);
				}
			}

//--------------------------------------------------------------------
// PENGUIN NPC's
//--------------------------------------------------------------------

	//- watches Rudi if he comes close --------------------------------
	
		void npc_pen_event ()
		{
			if (event_type == EVENT_IMPACT) {
				if (you == Rudi) {
					snd_pen_crashPlay();
				}
			}
		}

		void npc_penWatch ()
		{
			my.emask |= ENABLE_IMPACT;
			my.event = npc_pen_event;

			act_watchRudi();
	
			my.skill1 = 5;	//idle speed
			my.skill2 = 10;	//walk speed
			my.skill3 = 600;	//viewing range
		}
		
		void npc_penPatrol ()
		{
			my.skill1 = 5;
			my.skill2 = 10;
			my.skill3 = 400;
			my.skill4 = 6;
			my.skill5 = 12;

			effShadBlob(my, 4, "effShadPen.dds");

			act_patrolPath();
		}


//--------------------------------------------------------------------
// ESKIMO NPC's
//--------------------------------------------------------------------

	//- watches Rudi if he comes close --------------------------------
	
		void npc_esk_event ()
		{
			if (event_type == EVENT_IMPACT) {
				if (you == Rudi) {
					snd_esk_crashPlay();
				}
			}
		}

		void npc_eskWatch ()
		{
			my.emask |= ENABLE_IMPACT;
			my.event = npc_esk_event;

			act_watchRudi();

			my.skill1 = 5;	//idle speed
			my.skill2 = 7;	//walk speed
			my.skill3 = 700;	//viewing range
		}

	//- sits around ---------------------------------------------------

		void npc_eskSit ()
		{
			my.emask |= ENABLE_IMPACT;
			my.event = npc_esk_event;

			if (detail == DETAIL_LOW) {
				ent_animate(my, "sit", 0, 0);
				reset(my, DYNAMIC);
			} else {
				act_loop_anim("sit");
				my.skill1 = 10;
			}
		}
		
		void npc_eskPatrol ()
		{
			my.skill1 = 5;		//Idle-Animation
			my.skill2 = 9;		//Walk-Animation
			my.skill3 = 400;	//Sight distance
			my.skill4 = 5;		//Run time min
			my.skill5 = 10;	//Walk speed

			// Shadow
				effShadBlob(my, 5, "effShadEsk.dds");

			act_patrolPath();
		}

//--------------------------------------------------------------------
// ELF NPC's
//--------------------------------------------------------------------

	//- standing around while waving ----------------------------------

		void npc_elfWave ()
		{
			act_loop_anim("wave");
			my.skill1 = 10;
		}