//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : packets
// created (by) : Christian Behrenberg, 2007-10-19
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// PACKET FUNCTION
//--------------------------------------------------------------------
void packet ()
{
	// Reset assertion
		wait(1);

	// Initialization
		pk_init();

	// Create backup data
		reset_backupMe(my, packet);

	//	Waiting for activation
		while (is(my, INVISIBLE)) {
	
	   	// We were invisible and inactive -> reset
				if (reset_global) {
					reset_restoreMe(my);
					return;
				}
	
			wait(1);
		}

	// Main loop - waiting for being collected
		while (!reset_global) {
	
			if (pk_check()) {
				snd_pk_collect();
				break;
			}
	
			wait(1);
		}

	// If we were active and Rudi died -> reset
		if (reset_global) {
			reset_restoreMe(my);
			return;
		}

	// We "die"
		pk_remove();
}

	//-----------------------------------------------------------------
	// PACKET INITIALIZATION
	//-----------------------------------------------------------------
	void pk_init ()
	{
		//make me passable
			set(my, PASSABLE);

		//increase packet count
			pk_count++;
			pk_count_max++;

		//save ID
			my.ID = pk_count - 1;

		//register packet
			pk_ents[my.ID] = my;

		//not visible until we are switched on
			set(my, INVISIBLE);

		wait(1); //let all other packets register themselves

		//create a new packet, if there is no one
			if (pk_current == 0) {
				pk_current = pk_spawn();
			}
	}
	
//--------------------------------------------------------------------
// CHECK IF RUDI COLLECTS THE PACKET
//--------------------------------------------------------------------
int pk_check ()
{
	if (Rudi) {
		return(vec_dist(Rudi->x, my.x) <= pk_collectRange);
	} else {
		return(0);
	}
}

//--------------------------------------------------------------------
// SPAWNS A NEW PACKET
//--------------------------------------------------------------------
ENTITY* pk_spawn ()
{
	ENTITY* newPacket = pk_select();

 	if (newPacket) {
		pk_activate(newPacket);
	}

	return(newPacket);
}

//--------------------------------------------------------------------
// SELECTS A RANDOM PACKET FROM THE PACKET LIST
//--------------------------------------------------------------------
ENTITY* pk_select ()
{
	ENTITY *current;
	int randomNr = (int)(random(pk_count)); //take the n'th entity
	int i;

	//get through the list
	for (i = 0; i < pk_count_max; i++) {
		current = pk_ents[i];		//get entity
		if (current) {					//if entity is valid

			if (randomNr <= 0) {		//break if this is the n'th entity
				break;
			} else {
				randomNr--;				//we still have to search
			}
		}
	}
	
	// This is the selected entity.
	// If no one has been found, its 0!

		return(current);
}

//--------------------------------------------------------------------
// ACTIVATES A PACKET
//--------------------------------------------------------------------
void pk_activate (ENTITY* ent)
{
	reset(ent, INVISIBLE);
}

//--------------------------------------------------------------------
// REMOVES A PACKET
//--------------------------------------------------------------------
void pk_remove()
{
	//effect: spark
		eff_spawnEnt_verts(my, 20, eff_sparkMagic);

	//save my pointer
		//ENTITY* mySave = my;

	//de-register
		pk_ents[my.ID] = 0;
		pk_count--;

	//add sledges
		sl_addNr(1);

	//spawn new packet
		pk_current = pk_spawn();

	//remove entity
		//ent_remove(mySave);

	// Stay alive to be reset-enabled
		
		set(my, INVISIBLE);

	 	while (1) {

       	// We are collected -> reset
				if (reset_global) {
					reset_restoreMe(my);
				}

			wait(1);
		}
}

//--------------------------------------------------------------------
// PACKET ARROW
// Points to the next packet if it is outside of the screen
//--------------------------------------------------------------------
void pk_arrow_main ()
{
	//creates and initializes the arrow
		pk_arrow_init();

	//Main loop
	while (1) {
		
		//ROTATION

	   	//Doesn't work if Rudi and the packet aren't there!
	   	if ((Rudi != 0) && (pk_current != 0)) {

	      	// Angle from Rudi to packet
					vec_diff(vecTemp.x, pk_current->x, Rudi->x);
					vec_to_angle(vecTemp, vecTemp);
	
				// Rotation interpolation. Uses a special angle transformation
				// to project from entity rotation to view entity rotation.
					ang_lerp(pk_arrow->pan, pk_arrow->pan, vector(vecTemp.x + 90, 180, 0), pk_arr_rotBlend * time_step);
			}

		//POPUP
		
			if (Rudi) {

	      	//If packet is available and outside the screen -> popup
				if ((pk_current != 0) && (Rudi->playerDeath == 0)) {
					vec_set(vecTemp.x, pk_current->x);
					if (vec_to_screen(vecTemp.x, camera) == NULL) {
						vec_lerp(pk_arrow->scale_x, pk_arrow->scale_x, pk_arr_popup_scaleIn, pk_arr_popup_speedIn * time_step);
					} else {
						vec_lerp(pk_arrow->scale_x, pk_arrow->scale_x, pk_arr_popup_scaleOut, pk_arr_popup_speedOut * time_step);
					}
				} else {
					vec_lerp(pk_arrow->scale_x, pk_arrow->scale_x, pk_arr_popup_scaleOut, pk_arr_popup_speedOut * time_step);
				}
			}

		//VISIBILITY

			//if the arrow is too small, clip it
			if (pk_arrow->scale_x <= pk_arr_popup_clip) {
				set(pk_arrow, INVISIBLE);
			} else {
				reset(pk_arrow, INVISIBLE);
			}

		wait(1);
	}
}

	//-----------------------------------------------------------------
	// CREATES & INITIALIZES THE ARROW
	//-----------------------------------------------------------------
	void pk_arrow_init ()
	{
		//create arrow as view entity
			pk_arrow = ent_createlayer("arrow.mdl", 0, 1);

		//pose and orientate it
			vec_set(pk_arrow->x, vector(700, 0, -350));
			pk_arrow->tilt = 180; //see rotation code; init rotation
	}