//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : player
// created (by) : Christian Behrenberg, 2007-10-10
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// main player action
//--------------------------------------------------------------------

void pl_bridle ()
{
	you = (ENTITY*)(my.playerBridle);
	
	if (you) {

		if (my.sledgeFirst != NULL) {
			
			vec_set(you.x, my.x);
			vec_set(you.pan, my.pan);
			you.frame = my.frame;
			you.nextframe = my.nextframe;

			reset(you, INVISIBLE);

		} else {
			set(you, INVISIBLE);
		}
	}
}

void pl_rudi ()
{
	pl_rudi_init();

	while (my.playerDeath == 0)
	{
		if (key_9) {
			while (key_9) {wait(1);}
			lock = (++lock % 2);
		}
		
		if (key_space) {
			while (key_space) {wait(1);}
			sl_addNr(5);
		}

		cam_update();

		pl_turn();
		pl_gravity();

		if (lock) {
			hud_hide();
		} else {
			hud_show();
		}

		pl_move();
		pl_animate();
		pl_interpolate();
		pl_bridle();
		pl_log();
		
		vec_set(pl_listener, my.x);

		wait(1);
	}

	pl_death();

}

	//--------------------------------------------------------------------
	// player initialization
	//--------------------------------------------------------------------
	
	void pl_rudi_bridle ()
	{
		set(my, PASSABLE);
	}

	void pl_rudi_init ()
	{
		Rudi = my;				//indicate that I am Rudi!

		// Create reset data
			reset_backupMe(my, pl_rudi);

 		c_setminmax(my); 		//calculate real bounding box
		my.max_z *= 0.5;
		
		// Create bridle
			my.playerBridle = ent_create("rudiBridle.mdl", vector(0,0,-1000), pl_rudi_bridle);

		//set events for crashing
			my.emask |= (ENABLE_BLOCK | ENABLE_ENTITY);
			my.event = pl_event;

		pl_log();				//reset logging

		//create track logging datastructure
		if (trackRudi == 0) {
			trackRudi = create_TRACK(my.x, pl_track_points);
		}

		// Reset lifes if they are =0
			if (lifes <= 0) {
				lifes = lifes_resetNr;
				points = 0;
			}
			
		eff_rudiNoseGlow();
		pl_rudi_init_shad();
		
		//call game watchers
			game_watchPoints();
			game_watchLifes();
	}

	void pl_rudi_init_shad ()
	{
  		effShadSoft(Rudi, 313, 4);
  		effShadSoft(Rudi, 169, 4);
  		effShadSoft(Rudi, 323, 4);
  		effShadSoft(Rudi, 179, 4);
  		effShadSoft(Rudi, 27, 4);
  		effShadSoft(Rudi, 22, 4);
  		effShadSoft(Rudi, 18, 4);
	}

//--------------------------------------------------------------------
// player event
//--------------------------------------------------------------------
void pl_event ()
{
	//CRASH: running against entities or blocks
	if ((event_type == EVENT_BLOCK) || (event_type == EVENT_ENTITY)) {

		//crash against orthogonal walls (with a bit threshold)
		if (normal.z <= 0.2) {
			my.playerDeath = 1; //standard death: crashing
		}
	}
}

//--------------------------------------------------------------------
// turns the player
//--------------------------------------------------------------------
void pl_turn ()
{
	if (lock) {return;}

	//calculate movement direction
		vec_set(vecTemp, vector(RIGHT - LEFT, UP - DOWN, 0));
		vec_normalize(vecTemp, 1);

	//if we enter a valid direction, overwrite old movement vector
		if (vec_length(vecTemp) > 0) {
			vec_set(my.moveDir, vecTemp);
		}

	//interpolate angle
		vec_to_angle(vecTemp, my.moveDir);
		ang_lerp(vecTemp2, my.pan, vector(vecTemp.x, 0, 0), pl_turn_blendFac * time_step);
		my.pan = vecTemp2.x; //set only pan!
}

//--------------------------------------------------------------------
// checks, if the player is on earth or in air
//--------------------------------------------------------------------
void pl_gravity ()
{
	my.height = c_trace(my.x, vector(my.x, my.y, -2000), IGNORE_ME | IGNORE_PASSABLE) + my.min_z;
}

//--------------------------------------------------------------------
// moves the player
//--------------------------------------------------------------------
void pl_move ()
{
	// if move vector is valid, move
		if (vec_length(my.moveDir) > 0) {
			
			snd_pl_walkPlay();

			//create relative motionvector
			vec_set(vecTemp, vector(vec_length(my.moveDir), 0, 0));		//base motion
			vec_scale(vecTemp, pl_move_speed);									//motion speed
			vec_scale(vecTemp, time_step);										//time correction

			my.z += pl_move_marginOffset;											//raise for a better margin

			c_move(my, vecTemp, nullvector, IGNORE_PASSABLE | GLIDE);	//move
			my.z -= pl_move_marginOffset;											//revert margin raise
			
			pl_move_snowSplat();
		}

	// gravitation
		vec_set(vecTemp, vector(0, 0, -minv(my.height, pl_gravitation * time_step)));
		c_move(my, nullvector, vecTemp, IGNORE_PASSABLE | GLIDE);
		
	// if too low, die (e.g. water or emptyness)

   	if (my.z <= PL_DEATH_WATERHEIGTH) {
			my.playerDeath = 1;
		}
}

	void pl_move_snowSplat ()
	{
		if (detail == DETAIL_LOW) {return;}

		if (my.skill14 < 0) {

			vec_for_vertex(vecTemp, my, 169);
			effect(eff_snowSplat, 1, vecTemp, nullvector);

			vec_for_vertex(vecTemp, my, 313);
			effect(eff_snowSplat, 1, vecTemp, nullvector);

			vec_for_vertex(vecTemp, my, 179);
			effect(eff_snowSplat, 1, vecTemp, nullvector);

			vec_for_vertex(vecTemp, my, 323);
			effect(eff_snowSplat, 1, vecTemp, nullvector);

			my.skill14 += 3;

		} else {
			my.skill14 -= time_step;
		}
	}

//--------------------------------------------------------------------
// handles the whole character animation
//--------------------------------------------------------------------
void pl_animate ()
{
	//are we moving?
	if (vec_dist(my.logPos, my.x) > pl_anim_move_threshold * time_step) { //yes; This is the walk animation
	
   	//increase animation counter and clamp it 1t 100 percent
		my.animCounter = (my.animCounter + pl_anim_walk_speed * time_step) % 100;
		ent_animate(my, pl_anim_walk_frame, my.animCounter, ANM_CYCLE); //animate!

	} else {
		
   	//increase animation counter and clamp it 1t 100 percent
		my.animCounter = (my.animCounter + pl_anim_idle_speed * time_step) % 100;
		ent_animate(my, pl_anim_idle_frame, my.animCounter, ANM_CYCLE); //animate!

	}
}

//--------------------------------------------------------------------
// does some interpolation calculation to make the actor move and
// behave smmoothly
//--------------------------------------------------------------------
void pl_interpolate ()
{
	//INTERPOLATE MOTION ANGLE

		//interpolate only if we actually moved
		if (vec_length(my.moveDir) > 0) {

			vec_diff(vecTemp, my.x, my.logPos); //look from old pos to current position

			vec_set(vecTemp2, vecTemp); //copy; min. move dist will be checked on XY plane
			vecTemp2.z = 0;

			if (vec_length(vecTemp2) > pl_lerp_surfaceAng_threshold * time_step) { //min. XY plane movement

				vec_to_angle(vecTemp, vecTemp); //calc angle
				ang_lerp(my.pan, my.pan, vecTemp, pl_lerp_surfaceAng_fac * time_step); //interpolate angle
			}
		}
}

//--------------------------------------------------------------------
// logs some current player stats
//--------------------------------------------------------------------
void pl_log ()
{
	//INTERNAL LOGGING
		vec_set(my.logPos, my.x); //log position

	//TRACK RECORDING

   	// Log the position into the track determined by
		// equidistant intervals

		if (trackRudi != 0) {
	   	if (vec_dist(my.x, trackRudi->first->pos) > pl_track_logDist) {
				reg_TRACK_point(trackRudi, vector(my.x, my.y, my.z + my.min_z));
			}
		}
}

//--------------------------------------------------------------------
// executes the selected death method
//--------------------------------------------------------------------
void pl_death ()
{
	snd_pl_walkStop();

	switch (my.playerDeath) {

		case 1:	pl_death_crash();		//crashing (default)
					return;

		case 2:	pl_death_levelend();	//levelend
					return;

		//add other death types here
	}
}



	//-----------------------------------------------------------------
	// crash (standard death)
	//-----------------------------------------------------------------
	void pl_death_crash ()
	{
		//cut sledges
			sl_cut((ENTITY*)(my.sledgeFirst));

		var i;
		
		if (my.z <= PL_DEATH_WATERHEIGTH) {
			snd_pl_splash();
		} else {
			snd_pl_crash();
		}

		//play crash animation
		for (i = 0; i < 100; i += 7 * time_step) {
			pl_death_crash_cam ();
			ent_animate(my, "crash", i, 0); //crash

			// sink down in emptyness or water
				if (my.z <= PL_DEATH_WATERHEIGTH) {
					my.z -= 3 * time_step;
				}

			pl_bridle();

			wait(1);
		}
		
		//decrease lifes
			lifes--;

		var waitRestart = pl_death_stayTime * 16;
		while (waitRestart > 0) {
			pl_death_crash_cam ();
			waitRestart -= time_step;

			// sink down in emptyness or water
				if (my.z <= PL_DEATH_WATERHEIGTH) {
					my.z -= 3 * time_step;
				}
				
			pl_bridle();

			wait(1);
		}

		if (lifes == 0) {
			
			music_gameOver_start();

			menu_mode = MENU_MODE_GAMEOVER;

			//wait one frame to establish the Gameover screen
			wait(1);

			level_load("");

		} else {
			
			// Reset level

				reset_global = 1;			// Global reset indication
				my.deleteShadow = 1;
				lvl_reset();				// Reset level values
				wait(1);						// Wait for the resetted ent's
				reset_global = 0;			// Stop resetting

				// Remove bridle
					ent_remove((ENTITY*)(my.playerBridle));

            lvl_stopSnd();
				reset_restoreMe(my);		// Reset player
		}
	}

		void pl_death_crash_cam ()
		{
			cam_setOffset(cam_rudiDeath_offset);
			cam_setArc(cam_rudiDeath_arc);

			cam_update();
		}

	//-----------------------------------------------------------------
	// levelend (no death at all... but we stop "playing" now
	//-----------------------------------------------------------------
	void pl_death_levelend ()
	{
		//We need at least one sledge
		if (my.sledgeFirst != 0) {

			//Get hearts for sledges
			sl_getHearts((ENTITY*)(my.sledgeFirst));
		}
		
		wait(1);

      my.skill1 = pl_death_levelWaitTime * 16;
		while (my.skill1 > 0) {

			if (points_toAdd <= 0) {
				my.skill1 -= time_step;
			}

			pl_animate();
			pl_bridle();

			wait(1);
		}

      // This was the last level -> Credits
			if (lvl_current == lvl_count) {
				music_level_stop();
				lvl_stopSnd();
				cr_stoppable = 0;
				cr_load();
			}
	}