//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : shaders
// created (by) : Christian Behrenberg, 2007-11-12
//--------------------------------------------------------------------


//--------------------------------------------------------------------
// SHADER STARTUP
// Initializes & creates things on engine startup, e.g. cubemaps
//--------------------------------------------------------------------
void shader_startup ()
{
	// Cubemaps

		// Shiny shader cubemap
			fx_cube_shiny = bmap_create(FX_CUBE_SHINY);
			bmap_to_cubemap(fx_cube_shiny);

	// Skycubes

		// Nortpole
			fx_cube_sky_northpole = bmap_create(FX_CUBE_SKY_NORTHPOLE);
			bmap_to_cubemap(fx_cube_sky_northpole);
}

//--------------------------------------------------------------------
// SHADER: DETAIL MAP ON MODEL + SHINY
//--------------------------------------------------------------------
void fx_detailShiny ()
{
	my.material = mtl_detailShiny;
}

	//- initialization ------------------------------------------------
	void mtl_detailShiny_init ()
	{
		// Load cubemap
			mtl.skin1 = fx_cube_shiny;

		mtl.event = mtl_detailShiny_func;
	}

		//- shader event function --------------------------------------
		void mtl_detailShiny_func ()
		{
			// Updates the shader with the inversed camera view matrix
			// in order to display a correctly transformed environmental
			// mapping

			mat_set(mtl.matrix, matViewInv);

			mtl.matrix41 = 0;
			mtl.matrix42 = 0;
			mtl.matrix43 = 0;
		}

//--------------------------------------------------------------------
// SHADER: OCEAN
//
// params: cube - cubemap which should be used as reflection map
//--------------------------------------------------------------------
void fx_ocean (BMAP* cube)
{
	my.material = mtl_ocean;
	mtl_ocean_init(cube);
}

	//- initialization ------------------------------------------------
	void mtl_ocean_init (BMAP* cube)
	{
		// Assigns the cubemap
			mtl_ocean->skin2 = cube;
	}

//--------------------------------------------------------------------
// SHADER: DETAILMAP ON MODEL
//--------------------------------------------------------------------
void fx_detail ()
{
	my.material = mtl_detail;
}

//--------------------------------------------------------------------
// SHADER: SHINY
//
// params: cube - cubemap which should serve as reflection map
//--------------------------------------------------------------------
void fx_shiny (BMAP* cube)
{
	my.material = mtl_shiny;
	mtl_shiny_init(cube);
}

	//- initialization ------------------------------------------------
	void mtl_shiny_init (BMAP* cube)
	{
		// Assigns the cubemap
			mtl_shiny.skin1 = cube;

		mtl_shiny.event = mtl_shiny_func;
		set(mtl_shiny, ENABLE_VIEW);
	}

		//- shader event function --------------------------------------
		void mtl_shiny_func ()
		{
			mat_set(mtl.matrix,matViewInv);
			mtl.matrix41 = 0;
			mtl.matrix42 = 0;
			mtl.matrix43 = 0;
		}

//--------------------------------------------------------------------
// SHADER: WAVING GRASS
// Will be disabled if not in HIGH mode!
//--------------------------------------------------------------------
void fx_wavingGrass ()
{
	if (detail == DETAIL_HIGH) {
		my.material = mtl_wavingGrass;
	}
}

//--------------------------------------------------------------------
// SHADER: ALPHA TEST
// Alpha-test means: correct Z-Buffer sorting.
//--------------------------------------------------------------------
void fx_alpha ()
{
	my.material = mtl_alpha;
}