//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : sounds
// created (by) : Christian Behrenberg, 2007-12-09
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// LOADS ALL SOUND RESSOURCES
// Automatically on engine startup.
//--------------------------------------------------------------------
void snd_startup ()
{
	int i;
	char* buffer[64];

	// Single sounds
	
		// Game

			sndRudi_walk		= snd_create("sndRudi_walk.ogg");
			sndRudi_cut			= snd_create("sndRudi_cut.ogg");
			sndRudi_splash		= snd_create("sndRudi_splash.ogg");
			sndRudi_crash		= snd_create("sndRudi_crash.ogg");
			sndPacket_collect	= snd_create("sndPacket_collect.ogg");
			sndEnv_wind			= snd_create("sndEnv_wind.ogg");
			sndEnv_water		= snd_create("sndEnv_water.ogg");
			sndEnv_waterWind	= snd_create("sndEnv_waterWind.ogg");
			sndEnv_gulls		= snd_create("sndEnv_gulls.ogg");
			sndObj_fire			= snd_create("sndObj_fire.ogg");
			sndObj_gateBell	= snd_create("sndObj_gateBell.ogg");
			
		// Menu

			sndMenu_startGame	= snd_create("sndMenu_startGame.ogg");
			sndMenu_itemTouch = snd_create("sndMenu_itemTouch.ogg");
			sndMenu_itemClick = snd_create("sndMenu_itemClick.ogg");

	// Multiple sounds
	
		// Game

			// Eskimo shouts
				for (i = 0; i < SNDESK_CRASH_CNT; i++) {
					sprintf(buffer, "sndEsk_crash%d.ogg", i);
					sndEsk_crash[i] = snd_create(buffer);
				}

			// Penguin shouts
				for (i = 0; i < SNDPEN_CRASH_CNT; i++) {
					sprintf(buffer, "sndPen_crash%d.ogg", i);
					sndPen_crash[i] = snd_create(buffer);
				}
}

//--------------------------------------------------------------------
// SOUND EMITTER WHICH TAKES A VECTOR AS LISTENER POSITION
//
// Instead of 3D sounds which take the camera as listener source, this
// generator takes a referenced position vector as listener source.
// Instead of simply playing a sound, it takes two functions, will be
// executed if the vector is in the listener range.
//
// params: entMy    - entity which makes the sound
//         listener - referenced position vector
//         dist     - radius
//         playSnd  - function which will be executed once, when the
//                    vector is the first time in the radius
//         stopSnd  - function which will be executed once, when the
//                    vector is the first time outside the radius
//--------------------------------------------------------------------
void snd_emitByEntDist (ENTITY* entMy, VECTOR* listener, var dist, void* playSnd, void* stopSnd)
{
	void (*func)(void); // Function pointer

	// Take the entMy as my entity
		my = entMy;

	// Main loop
	while (1) {

		// If vector is "near"
		if (vec_dist(my.x, listener) < dist) {

			// Execute play function once
				func = playSnd;
				func();

			// Wait until vector is "far"
			while (1) {

				// Stop when outside
					if (vec_dist(my.x, listener) > dist) {break;}

				// Play sound repeatedly to restart if
				// sounds have been stopeed by snd_stopAll
					func = playSnd;
					func();

				wait(1);
			}

			// Vector is "far", execute stop function once
				func = stopSnd;
				func();
		}

		wait(1);
	}
}

//--------------------------------------------------------------------
// STOPS ALL SOUND FUNCTIONS FOR THE LEVEL
//--------------------------------------------------------------------
void lvl_stopSnd ()
{
	snd_pl_walkStop();			// Rudi's walk
	snd_env_windStop();			// Wind
	snd_env_waterStop();			// Water
	snd_env_waterWindStop();	// Seashore
	snd_obj_fireStop();			// Fire crackers
	snd_env_gullsStop();			// Seagulls
}

//--------------------------------------------------------------------
// PLAYER SOUNDS
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// Rudi's walk
	//-----------------------------------------------------------------

		void snd_pl_walkPlay ()
		{
			if ((sndRudi_walkHndl==0) || (snd_playing(sndRudi_walkHndl) == 0)) {
				sndRudi_walkHndl = snd_loop(sndRudi_walk, volSfx, 0);
			}
		}
		
		void snd_pl_walkStop ()
		{
			snd_stop(sndRudi_walkHndl);
			sndRudi_walkHndl = NULL;
		}
		
	//-----------------------------------------------------------------
	// Rudi cuts his chain
	//-----------------------------------------------------------------

		void snd_pl_cut ()
		{
			snd_play(sndRudi_cut, volSfx, 0);
		}

	//-----------------------------------------------------------------
	// Rudi jumps into water
	//-----------------------------------------------------------------

		void snd_pl_splash ()
		{
			snd_play(sndRudi_splash, volSfx, 0);
		}
		
	//-----------------------------------------------------------------
	// Rudi hits an obstacle (crash)
	//-----------------------------------------------------------------

		void snd_pl_crash ()
		{
			snd_play(sndRudi_crash, volSfx, 0);
		}

	//-----------------------------------------------------------------
	// Rudi collects a packet
	//-----------------------------------------------------------------

		void snd_pk_collect ()
		{
			snd_play(sndPacket_collect, volSfx, 0);
		}
		
//--------------------------------------------------------------------
// ENVIRONMENTAL SOUNDS
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// Wind
	//-----------------------------------------------------------------

		void snd_env_windPlay ()
		{
			if ((sndEnv_windHndl==0) || (snd_playing(sndEnv_windHndl) == 0)) {
				sndEnv_windHndl = snd_loop(sndEnv_wind, volSfx, 0);
			}
		}
		
		void snd_env_windStop ()
		{
			snd_stop(sndEnv_windHndl);
			sndEnv_windHndl = NULL;
		}

	//-----------------------------------------------------------------
	// Water
	//-----------------------------------------------------------------

		void snd_env_waterPlay ()
		{
			if ((sndEnv_waterHndl==0) || (snd_playing(sndEnv_waterHndl) == 0)) {
				sndEnv_waterHndl = snd_loop(sndEnv_water, volSfx, 0);
			}
		}
		
		void snd_env_waterStop ()
		{
			snd_stop(sndEnv_waterHndl);
			sndEnv_waterHndl = NULL;
		}
		
	//-----------------------------------------------------------------
	// Seashore
	//-----------------------------------------------------------------

		void snd_env_waterWindPlay ()
		{
			if ((sndEnv_waterWindHndl==0) || (snd_playing(sndEnv_waterWindHndl) == 0)) {
				sndEnv_waterWindHndl = snd_loop(sndEnv_waterWind, volSfx, 0);
			}
		}
		
		void snd_env_waterWindStop ()
		{
			snd_stop(sndEnv_waterWindHndl);
			sndEnv_waterWindHndl = NULL;
		}
		
	//-----------------------------------------------------------------
	// Seagulls
	//-----------------------------------------------------------------

		void snd_env_gullsPlay ()
		{
			if ((sndEnv_gullsHndl==0) || (snd_playing(sndEnv_gullsHndl) == 0)) {
				sndEnv_gullsHndl = snd_loop(sndEnv_gulls, volSfx, 0);
			}
		}
		
		void snd_env_gullsStop ()
		{
			snd_stop(sndEnv_gullsHndl);
			sndEnv_gullsHndl = NULL;
		}
		
//--------------------------------------------------------------------
// OBJECT SOUNDS
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// Fire
	//-----------------------------------------------------------------

		void snd_obj_firePlay ()
		{
			if ((sndObj_fireHndl==0) || (snd_playing(sndObj_fireHndl) == 0)) {
				sndObj_fireHndl = snd_loop(sndObj_fire, volSfx, 0);
			}
		}
		
		void snd_obj_fireStop ()
		{
			snd_stop(sndObj_fireHndl);
			sndObj_fireHndl = NULL;
		}
		
	//-----------------------------------------------------------------
	// Christmas-Gate bell
	//-----------------------------------------------------------------

		void snd_obj_gateBellPlay ()
		{
			snd_play(sndObj_gateBell, volSfx, 0);
		}
		
//--------------------------------------------------------------------
// NPC SOUNDS
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// Eskimo shout
	//-----------------------------------------------------------------

		int snd_esk_crashPlay_time;
		void snd_esk_crashPlay ()
		{
			if (snd_esk_crashPlay_time != total_frames) {
				snd_esk_crashPlay_time = total_frames;
			} else {
				return;
			}
		
			sndEsk_crash_current = (sndEsk_crash_current + 1) % SNDESK_CRASH_CNT;
			snd_play(sndEsk_crash[sndEsk_crash_current], volSfx, 0);
		}

	//-----------------------------------------------------------------
	// Penguin shout
	//-----------------------------------------------------------------

		int snd_pen_crashPlay_time;
		void snd_pen_crashPlay ()
		{
			if (snd_pen_crashPlay_time != total_frames) {
				snd_pen_crashPlay_time = total_frames;
			} else {
				return;
			}
		
			sndPen_crash_current = (sndPen_crash_current + 1) % SNDPEN_CRASH_CNT;
			snd_play(sndPen_crash[sndPen_crash_current], volSfx, 0);
		}
		
//--------------------------------------------------------------------
// MENU SOUNDS
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// Menu: user clicks "Start Game"
	//-----------------------------------------------------------------

		void snd_menu_startGamePlay ()
		{
			snd_play(sndMenu_startGame, volSfx, 0);
		}

	//-----------------------------------------------------------------
	// Menu: user touches an item
	//-----------------------------------------------------------------

		void snd_menu_itemTouchPlay ()
		{
			snd_play(sndMenu_itemTouch, volSfx, 0);
		}
		
	//-----------------------------------------------------------------
	// Menu: user clicks an item
	//-----------------------------------------------------------------

		void snd_menu_itemClickPlay ()
		{
			snd_play(sndMenu_itemClick, volSfx, 0);
		}