//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : splashscreens
// created (by) : Christian Behrenberg, 2007-11-03
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// SHOWS THE SPLASHSCREEN SEQUENCE
// The global variable splash_ready will be set to 1 after finishing
// for external evaluation
//--------------------------------------------------------------------
void splash_show ()
{
	splash_init ();			// Load all ressources
	wait(3);						// Tripple buffering

	splash_ready = 0;			// Reset global flag
	music_splash_start();	// Start music

	// Show splashscreens

		int i;
		var timer;
		
		// Iterate through all splashscreens
		for (i=0; i < SPLASH_SCREENS_CNT; i++) {

			// Make panel visible and assign splash bitmap
				set(splashPanel, VISIBLE);
				splashPanel->bmap = splash_bmaps[i];

			// Wait for next splashscreen

				timer = SPLASH_SCREEN_TIME[i] * SECOND; // Reset timer

				while (timer > 0) {

					// Skipping
					if (key_any) {						// User wants to skip
						while (key_any) {wait(1);} // Wait until key is released
						i = SPLASH_SCREENS_CNT;		// Skip splashscreens
						break;							// Stop now
					}

					timer -= time_step;
					wait(1);
				}
		}

	// Show Video

 		// Prepare target bitmap
			splashPanel->bmap = splash_video; // Assign bitmap

			// Scale to fit fullscreen
				splashPanel->scale_x = screen_size.x / bmap_width(splash_video);
				splashPanel->scale_y = screen_size.y / bmap_height(splash_video);

			set(splashPanel, FILTER); // No unfiltered playback

		// Start video stream
			var videoHandle = media_play(SPLASH_VIDEO_FILE, splashPanel->bmap, 100);

		// Wait until video is finished
			while (media_playing(videoHandle)) {

				// Skipping
				if (key_any) {						// User wants to skip
					while (key_any) {wait(1);} // Wait until key is released
					media_stop(videoHandle);	// Stop video
					break;							// Stop now
				}

				wait(1);
			}

	splash_free();			// Remove ressources
	music_splash_stop();	// Stop music
	splash_ready = 1;		// Finished. Activate association flag
}

	//- loads all ressources ------------------------------------------
	void splash_init ()
	{
		//	Load bitmap ressources
			int i; char buffer[128];
			for (i=0; i < SPLASH_SCREENS_CNT; i++) {
				sprintf(buffer, "splash%c.tga", 'A' + i);
				splash_bmaps[i] = bmap_create(buffer);
			}

		// Create video target
			splash_video = bmap_createblack(SPLASH_VIDEO_SIZE[0], SPLASH_VIDEO_SIZE[1], 24);

		// Create panel for the splashes
			splashPanel = pan_create("", 999);
	}

	//- frees all ressources ------------------------------------------
	void splash_free ()
	{
		// Make panel and current bitmap invisible
			reset(splashPanel, VISIBLE);
			
		// Remove splashscreens
			int i;
			for (i = 0; i < SPLASH_SCREENS_CNT; i++) {
				bmap_remove(splash_bmaps[i]);
			}
			
		// Remove video target
			bmap_remove(splash_video);

		// Remove Panel
			pan_remove(splashPanel);
	}