//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : resolution independent display
// created (by) : Christian Behrenberg, 2007-11-04
//
// description  : A collection of functions which can be used in
//                combination of native panels to create a resolution
//                independent display (important for menu's, 2D games,
//                HUDs, etc.). The most important function is
//                ri_refreshPanel which will correctly transform a
//                for *one* frame. You have to define a specific
//                resolution which will be called "reference resolution
//                because all position information have to be in this
//                scope. The functions will the correctly transform
//                the panels into the scope of the current screensize.
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// TRANSFORM A REFERENCE SCALE
// In is represented in the scope of reference but should be repre-
// sented in the scope of a current measure.
//
// params:  in             - incoming value
//          reference      - reference scale
//          currentMeasure - transformation reference
//
// returns: in scaled into the scope of currentMeasure
//--------------------------------------------------------------------
var ri (var in, var reference, var currentMeasure)
{
	if (in != 0) {
		return((currentMeasure / reference) * in);
	} else {
		return(1);
	}
}

	//--------------------------------------------------------------------
	// CONVENIENCE FUNCTIONS
	// Transform an incoming variable in the scope of the screen axis'
	//--------------------------------------------------------------------
	var ri_x (var in) {return(ri(in, RI_REFRES_X, screen_size.x));}
	var ri_y (var in) {return(ri(in, RI_REFRES_Y, screen_size.y));}

//--------------------------------------------------------------------
// REFRESHES A PANEL
//
// params: panel - panel which will be refreshed
//         pos   - pos in the scope of the reference scale
//--------------------------------------------------------------------
void ri_refreshPanel (PANEL* panel, VECTOR* pos)
{
	if (panel) {
		
		// Annotation: in contrast to the workshop I added here a
		// a slight shift to the upper left corner for a reason: when
		// the function poses the panel, there could be a unexpected
		// shift to the lower rigth caused by different value res-
		// olutions during divisions and multiplications. So a 
		// preventative shift will fix the issue, that levels are
		// visible through a slight line in the upper left row/col
		// when using a fullscreen panel (e.g. the ingame cfg screen).

		// Pose panel
			panel->pos_x = ri_x(pos->x) - 0.05;
			panel->pos_y = ri_y(pos->y) - 0.05;

		// Scale panel
			panel->scale_x = ri_x(1);
			panel->scale_y = ri_y(1);
			
		// Enable filtering
			set(panel, FILTER);
	}
}