//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : system
// created (by) : Christian Behrenberg, 2007-10-10
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// SYSTEM START
// Initializes the system and calls the main system loop.
//--------------------------------------------------------------------
void sys_main()
{
 	sys_init();	// Initialization
	sys_loop();	// Main loop
}

	//--------------------------------------------------------------------
	// SYSTEM INITIALIZATION
	// Resets the random generator and initializes the video and the key
	// settings.
	//--------------------------------------------------------------------
	void sys_init()
	{
		// Seed the random generator
			randomize();

		// System initializations

			sys_video_init(); // Video
			sys_keys_init();	// Keys
	}
	
		//- VIDEO ---------------------------------------------------------
		void sys_video_init ()
		{
      	// No tearing, when no level is loaded
				vec_set(screen_color, vector(5, 5, 5));

         // Init video adapter
				video_switch(DEF_VIDEO_RES, DEF_VIDEO_DEPTH, DEF_VIDEO_MODE);
				
			// Set window title
				video_window(NULL, NULL, 0, DEF_WINDOW_TITLE);
		}
		
		//- key mappings --------------------------------------------------
		void sys_keys_init ()
		{
			// Disable default key bindings and set new bindings

				on_f2		= sys_key_null;
				on_f3		= sys_key_null;
				on_f4		= sys_key_null;
				on_f5		= sys_key_null;
				on_f8		= sys_key_null;
				on_esc	= sys_key_null;
				on_close = sys_close;

				#ifndef COMPILE
					// In development mode, freeze game with F8
						on_f8	= sys_key_freeze;
				#endif
		}

			// Dummy key bind
				void sys_key_null () {wait(1);}
			
	//--------------------------------------------------------------------
	// SYSTEM LOOP
	// Runs each frame specific system functions.
	//--------------------------------------------------------------------
	void sys_loop()
	{
		while (1) {

			sys_keys_check();	//Check keys

			wait(1);
		}
	}

		//- KEY CHECK --------------------------------------------------------
		void sys_keys_check ()
		{
			// Converts multiple input data into abstract key
			// information.

				UP		= ((key_cuu || key_w) || (joy_force.y > 0));
				DOWN	= ((key_cud || key_s) || (joy_force.y < 0));
				LEFT	= ((key_cul || key_a) || (joy_force.x < 0));
				RIGHT	= ((key_cur || key_d) || (joy_force.x > 0));
		}

		//- TOGGLE FREEZE ----------------------------------------------------
		void sys_key_freeze ()
		{
			// Toggles the freeze mode
				freeze_mode = !freeze_mode;
		}

//--------------------------------------------------------------------
// SHUTDOWN
// Executes a row of functions when the engine gets closed.
//--------------------------------------------------------------------
void sys_close()
{
	// Remove Rudi's TRAC object
		remove_TRACK(trackRudi);

	// Shut down the engine
		sys_exit("");
}